@@ -20,19 +20,27 @@ import (
 	"testing"
 	"time"
 
-	"github.com/golang/protobuf/proto"
-	"github.com/golang/protobuf/ptypes"
+	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	dto "github.com/prometheus/client_model/go"
+
+	"github.com/prometheus/common/model"
 )
 
 func TestCreateOpenMetrics(t *testing.T) {
-	openMetricsTimestamp, err := ptypes.TimestampProto(time.Unix(12345, 600000000))
-	if err != nil {
+	openMetricsTimestamp := timestamppb.New(time.Unix(12345, 600000000))
+	if err := openMetricsTimestamp.CheckValid(); err != nil {
 		t.Error(err)
 	}
 
-	var scenarios = []struct {
+	oldDefaultScheme := model.NameEscapingScheme
+	model.NameEscapingScheme = model.NoEscaping
+	defer func() {
+		model.NameEscapingScheme = oldDefaultScheme
+	}()
+
+	scenarios := []struct {
 		in  *dto.MetricFamily
 		out string
 	}{
@@ -43,13 +51,13 @@ func TestCreateOpenMetrics(t *testing.T) {
 				Help: proto.String("two-line\n doc  str\\ing"),
 				Type: dto.MetricType_COUNTER.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("labelname"),
 								Value: proto.String("val1"),
 							},
-							&dto.LabelPair{
+							{
 								Name:  proto.String("basename"),
 								Value: proto.String("basevalue"),
 							},
@@ -58,13 +66,13 @@ func TestCreateOpenMetrics(t *testing.T) {
 							Value: proto.Float64(42),
 						},
 					},
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("labelname"),
 								Value: proto.String("val2"),
 							},
-							&dto.LabelPair{
+							{
 								Name:  proto.String("basename"),
 								Value: proto.String("basevalue"),
 							},
@@ -82,20 +90,92 @@ name{labelname="val1",basename="basevalue"} 42.0
 name{labelname="val2",basename="basevalue"} 0.23 1.23456789e+06
 `,
 		},
-		// 1: Gauge, some escaping required, +Inf as value, multi-byte characters in label values.
+		// 1: Dots in name
+		{
+			in: &dto.MetricFamily{
+				Name: proto.String("name.with.dots"),
+				Help: proto.String("boring help"),
+				Type: dto.MetricType_COUNTER.Enum(),
+				Metric: []*dto.Metric{
+					{
+						Label: []*dto.LabelPair{
+							{
+								Name:  proto.String("labelname"),
+								Value: proto.String("val1"),
+							},
+							{
+								Name:  proto.String("basename"),
+								Value: proto.String("basevalue"),
+							},
+						},
+						Counter: &dto.Counter{
+							Value: proto.Float64(42),
+						},
+					},
+					{
+						Label: []*dto.LabelPair{
+							{
+								Name:  proto.String("labelname"),
+								Value: proto.String("val2"),
+							},
+							{
+								Name:  proto.String("basename"),
+								Value: proto.String("basevalue"),
+							},
+						},
+						Counter: &dto.Counter{
+							Value: proto.Float64(.23),
+						},
+						TimestampMs: proto.Int64(1234567890),
+					},
+				},
+			},
+			out: `# HELP "name.with.dots" boring help
+# TYPE "name.with.dots" unknown
+{"name.with.dots",labelname="val1",basename="basevalue"} 42.0
+{"name.with.dots",labelname="val2",basename="basevalue"} 0.23 1.23456789e+06
+`,
+		},
+		// 2: Dots in name, no labels
+		{
+			in: &dto.MetricFamily{
+				Name: proto.String("name.with.dots"),
+				Help: proto.String("boring help"),
+				Type: dto.MetricType_COUNTER.Enum(),
+				Metric: []*dto.Metric{
+					{
+						Counter: &dto.Counter{
+							Value: proto.Float64(42),
+						},
+					},
+					{
+						Counter: &dto.Counter{
+							Value: proto.Float64(.23),
+						},
+						TimestampMs: proto.Int64(1234567890),
+					},
+				},
+			},
+			out: `# HELP "name.with.dots" boring help
+# TYPE "name.with.dots" unknown
+{"name.with.dots"} 42.0
+{"name.with.dots"} 0.23 1.23456789e+06
+`,
+		},
+		// 3: Gauge, some escaping required, +Inf as value, multi-byte characters in label values.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("gauge_name"),
 				Help: proto.String("gauge\ndoc\nstr\"ing"),
 				Type: dto.MetricType_GAUGE.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_1"),
 								Value: proto.String("val with\nnew line"),
 							},
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_2"),
 								Value: proto.String("val with \\backslash and \"quotes\""),
 							},
@@ -104,13 +184,13 @@ name{labelname="val2",basename="basevalue"} 0.23 1.23456789e+06
 							Value: proto.Float64(math.Inf(+1)),
 						},
 					},
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_1"),
 								Value: proto.String("Björn"),
 							},
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_2"),
 								Value: proto.String("佖佥"),
 							},
@@ -127,20 +207,65 @@ gauge_name{name_1="val with\nnew line",name_2="val with \\backslash and \"quotes
 gauge_name{name_1="Björn",name_2="佖佥"} 3.14e+42
 `,
 		},
-		// 2: Unknown, no help, one sample with no labels and -Inf as value, another sample with one label.
+		// 4: Gauge, utf-8, some escaping required, +Inf as value, multi-byte characters in label values.
+		{
+			in: &dto.MetricFamily{
+				Name: proto.String("gauge.name\""),
+				Help: proto.String("gauge\ndoc\nstr\"ing"),
+				Type: dto.MetricType_GAUGE.Enum(),
+				Metric: []*dto.Metric{
+					{
+						Label: []*dto.LabelPair{
+							{
+								Name:  proto.String("name.1"),
+								Value: proto.String("val with\nnew line"),
+							},
+							{
+								Name:  proto.String("name*2"),
+								Value: proto.String("val with \\backslash and \"quotes\""),
+							},
+						},
+						Gauge: &dto.Gauge{
+							Value: proto.Float64(math.Inf(+1)),
+						},
+					},
+					{
+						Label: []*dto.LabelPair{
+							{
+								Name:  proto.String("name.1"),
+								Value: proto.String("Björn"),
+							},
+							{
+								Name:  proto.String("name*2"),
+								Value: proto.String("佖佥"),
+							},
+						},
+						Gauge: &dto.Gauge{
+							Value: proto.Float64(3.14e42),
+						},
+					},
+				},
+			},
+			out: `# HELP "gauge.name\"" gauge\ndoc\nstr\"ing
+# TYPE "gauge.name\"" gauge
+{"gauge.name\"","name.1"="val with\nnew line","name*2"="val with \\backslash and \"quotes\""} +Inf
+{"gauge.name\"","name.1"="Björn","name*2"="佖佥"} 3.14e+42
+`,
+		},
+		// 5: Unknown, no help, one sample with no labels and -Inf as value, another sample with one label.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("unknown_name"),
 				Type: dto.MetricType_UNTYPED.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Untyped: &dto.Untyped{
 							Value: proto.Float64(math.Inf(-1)),
 						},
 					},
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_1"),
 								Value: proto.String("value 1"),
 							},
@@ -156,40 +281,40 @@ unknown_name -Inf
 unknown_name{name_1="value 1"} -1.23e-45
 `,
 		},
-		// 3: Summary.
+		// 6: Summary.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("summary_name"),
 				Help: proto.String("summary docstring"),
 				Type: dto.MetricType_SUMMARY.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Summary: &dto.Summary{
 							SampleCount: proto.Uint64(42),
 							SampleSum:   proto.Float64(-3.4567),
 							Quantile: []*dto.Quantile{
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.5),
 									Value:    proto.Float64(-1.23),
 								},
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.9),
 									Value:    proto.Float64(.2342354),
 								},
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.99),
 									Value:    proto.Float64(0),
 								},
 							},
 						},
 					},
-					&dto.Metric{
+					{
 						Label: []*dto.LabelPair{
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_1"),
 								Value: proto.String("value 1"),
 							},
-							&dto.LabelPair{
+							{
 								Name:  proto.String("name_2"),
 								Value: proto.String("value 2"),
 							},
@@ -198,15 +323,15 @@ unknown_name{name_1="value 1"} -1.23e-45
 							SampleCount: proto.Uint64(4711),
 							SampleSum:   proto.Float64(2010.1971),
 							Quantile: []*dto.Quantile{
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.5),
 									Value:    proto.Float64(1),
 								},
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.9),
 									Value:    proto.Float64(2),
 								},
-								&dto.Quantile{
+								{
 									Quantile: proto.Float64(0.99),
 									Value:    proto.Float64(3),
 								},
@@ -229,35 +354,35 @@ summary_name_sum{name_1="value 1",name_2="value 2"} 2010.1971
 summary_name_count{name_1="value 1",name_2="value 2"} 4711
 `,
 		},
-		// 4: Histogram
+		// 7: Histogram
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("request_duration_microseconds"),
 				Help: proto.String("The response latency."),
 				Type: dto.MetricType_HISTOGRAM.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Histogram: &dto.Histogram{
 							SampleCount: proto.Uint64(2693),
 							SampleSum:   proto.Float64(1756047.3),
 							Bucket: []*dto.Bucket{
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(100),
 									CumulativeCount: proto.Uint64(123),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(120),
 									CumulativeCount: proto.Uint64(412),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(144),
 									CumulativeCount: proto.Uint64(592),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(172.8),
 									CumulativeCount: proto.Uint64(1524),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(math.Inf(+1)),
 									CumulativeCount: proto.Uint64(2693),
 								},
@@ -277,31 +402,31 @@ request_duration_microseconds_sum 1.7560473e+06
 request_duration_microseconds_count 2693
 `,
 		},
-		// 5: Histogram with missing +Inf bucket.
+		// 8: Histogram with missing +Inf bucket.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("request_duration_microseconds"),
 				Help: proto.String("The response latency."),
 				Type: dto.MetricType_HISTOGRAM.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Histogram: &dto.Histogram{
 							SampleCount: proto.Uint64(2693),
 							SampleSum:   proto.Float64(1756047.3),
 							Bucket: []*dto.Bucket{
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(100),
 									CumulativeCount: proto.Uint64(123),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(120),
 									CumulativeCount: proto.Uint64(412),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(144),
 									CumulativeCount: proto.Uint64(592),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(172.8),
 									CumulativeCount: proto.Uint64(1524),
 								},
@@ -321,28 +446,28 @@ request_duration_microseconds_sum 1.7560473e+06
 request_duration_microseconds_count 2693
 `,
 		},
-		// 6: Histogram with missing +Inf bucket but with different exemplars.
+		// 9: Histogram with missing +Inf bucket but with different exemplars.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("request_duration_microseconds"),
 				Help: proto.String("The response latency."),
 				Type: dto.MetricType_HISTOGRAM.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Histogram: &dto.Histogram{
 							SampleCount: proto.Uint64(2693),
 							SampleSum:   proto.Float64(1756047.3),
 							Bucket: []*dto.Bucket{
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(100),
 									CumulativeCount: proto.Uint64(123),
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(120),
 									CumulativeCount: proto.Uint64(412),
 									Exemplar: &dto.Exemplar{
 										Label: []*dto.LabelPair{
-											&dto.LabelPair{
+											{
 												Name:  proto.String("foo"),
 												Value: proto.String("bar"),
 											},
@@ -351,16 +476,16 @@ request_duration_microseconds_count 2693
 										Timestamp: openMetricsTimestamp,
 									},
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(144),
 									CumulativeCount: proto.Uint64(592),
 									Exemplar: &dto.Exemplar{
 										Label: []*dto.LabelPair{
-											&dto.LabelPair{
+											{
 												Name:  proto.String("foo"),
 												Value: proto.String("baz"),
 											},
-											&dto.LabelPair{
+											{
 												Name:  proto.String("dings"),
 												Value: proto.String("bums"),
 											},
@@ -368,7 +493,7 @@ request_duration_microseconds_count 2693
 										Value: proto.Float64(140.14),
 									},
 								},
-								&dto.Bucket{
+								{
 									UpperBound:      proto.Float64(172.8),
 									CumulativeCount: proto.Uint64(1524),
 								},
@@ -388,14 +513,14 @@ request_duration_microseconds_sum 1.7560473e+06
 request_duration_microseconds_count 2693
 `,
 		},
-		// 7: Simple Counter.
+		// 10: Simple Counter.
 		{
 			in: &dto.MetricFamily{
 				Name: proto.String("foos_total"),
 				Help: proto.String("Number of foos."),
 				Type: dto.MetricType_COUNTER.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Counter: &dto.Counter{
 							Value: proto.Float64(42),
 						},
@@ -407,7 +532,7 @@ request_duration_microseconds_count 2693
 foos_total 42.0
 `,
 		},
-		// 8: No metric.
+		// 11: No metric.
 		{
 			in: &dto.MetricFamily{
 				Name:   proto.String("name_total"),
@@ -441,7 +566,6 @@ foos_total 42.0
 			)
 		}
 	}
-
 }
 
 func BenchmarkOpenMetricsCreate(b *testing.B) {
@@ -450,17 +574,17 @@ func BenchmarkOpenMetricsCreate(b *testing.B) {
 		Help: proto.String("The response latency."),
 		Type: dto.MetricType_HISTOGRAM.Enum(),
 		Metric: []*dto.Metric{
-			&dto.Metric{
+			{
 				Label: []*dto.LabelPair{
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_1"),
 						Value: proto.String("val with\nnew line"),
 					},
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_2"),
 						Value: proto.String("val with \\backslash and \"quotes\""),
 					},
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_3"),
 						Value: proto.String("Just a quite long label value to test performance."),
 					},
@@ -469,40 +593,40 @@ func BenchmarkOpenMetricsCreate(b *testing.B) {
 					SampleCount: proto.Uint64(2693),
 					SampleSum:   proto.Float64(1756047.3),
 					Bucket: []*dto.Bucket{
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(100),
 							CumulativeCount: proto.Uint64(123),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(120),
 							CumulativeCount: proto.Uint64(412),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(144),
 							CumulativeCount: proto.Uint64(592),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(172.8),
 							CumulativeCount: proto.Uint64(1524),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(math.Inf(+1)),
 							CumulativeCount: proto.Uint64(2693),
 						},
 					},
 				},
 			},
-			&dto.Metric{
+			{
 				Label: []*dto.LabelPair{
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_1"),
 						Value: proto.String("Björn"),
 					},
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_2"),
 						Value: proto.String("佖佥"),
 					},
-					&dto.LabelPair{
+					{
 						Name:  proto.String("name_3"),
 						Value: proto.String("Just a quite long label value to test performance."),
 					},
@@ -511,19 +635,19 @@ func BenchmarkOpenMetricsCreate(b *testing.B) {
 					SampleCount: proto.Uint64(5699),
 					SampleSum:   proto.Float64(49484343543.4343),
 					Bucket: []*dto.Bucket{
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(100),
 							CumulativeCount: proto.Uint64(120),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(120),
 							CumulativeCount: proto.Uint64(412),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(144),
 							CumulativeCount: proto.Uint64(596),
 						},
-						&dto.Bucket{
+						{
 							UpperBound:      proto.Float64(172.8),
 							CumulativeCount: proto.Uint64(1535),
 						},
@@ -545,7 +669,7 @@ func BenchmarkOpenMetricsCreate(b *testing.B) {
 }
 
 func TestOpenMetricsCreateError(t *testing.T) {
-	var scenarios = []struct {
+	scenarios := []struct {
 		in  *dto.MetricFamily
 		err string
 	}{
@@ -555,7 +679,7 @@ func TestOpenMetricsCreateError(t *testing.T) {
 				Help: proto.String("doc string"),
 				Type: dto.MetricType_UNTYPED.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Untyped: &dto.Untyped{
 							Value: proto.Float64(math.Inf(-1)),
 						},
@@ -571,7 +695,7 @@ func TestOpenMetricsCreateError(t *testing.T) {
 				Help: proto.String("doc string"),
 				Type: dto.MetricType_COUNTER.Enum(),
 				Metric: []*dto.Metric{
-					&dto.Metric{
+					{
 						Untyped: &dto.Untyped{
 							Value: proto.Float64(math.Inf(-1)),
 						},
@@ -596,5 +720,4 @@ func TestOpenMetricsCreateError(t *testing.T) {
 			)
 		}
 	}
-
 }
